@@ -28,7 +28,7 @@
 
 #include <cmath>
 #include <QDebug>
-#include <QFile>
+#include "private/export3d.h"
 
 #ifdef __APPLE__
 #include <OpenGL/gl.h>
@@ -44,8 +44,6 @@
 using namespace std;
 using namespace Analitza;
 
-Q_DECLARE_METATYPE(PlotItem*)
-
 void qgluPerspective(GLdouble fovy, GLdouble aspect, GLdouble zNear, GLdouble zFar)
 {
     const GLdouble ymax = zNear * tan(fovy * M_PI / 360.0);
@@ -303,82 +301,12 @@ void Plotter3D::drawPlots()
     glDisable(GL_LIGHT0);
 }
 
-template <typename T>
-static QByteArray fromNumbers(const QVector<T>& input)
-{
-    QByteArray ret;
-    foreach(qreal r, input) {
-        ret += QByteArray::number(r)+QByteArrayLiteral(" ");
-    }
-    ret.chop(1);
-    return ret;
-}
-
-static QByteArray fromNumbers(const QVector<QVector3D>& input)
-{
-    QByteArray ret;
-    foreach(const QVector3D &r, input) {
-        ret += QByteArray::number(r.x())+QByteArrayLiteral(" ");
-        ret += QByteArray::number(r.y())+QByteArrayLiteral(" ");
-        ret += QByteArray::number(r.z())+QByteArrayLiteral(" ");
-    }
-    ret.chop(1);
-    return ret;
-}
-
-static QVector<int> makeTriangles(const QVector<uint>& input)
-{
-    QVector<int> ret;
-    int i = 0;
-    foreach(uint val, input) {
-        ret += val;
-        if(i==2) {
-            ret += -1;
-            i = 0;
-        } else
-            ++i;
-    }
-    ret += -1;
-    return ret;
-}
-
 void Plotter3D::exportSurfaces(const QString& path) const
 {
-    QFile f(path);
-    bool b = f.open(QIODevice::WriteOnly | QIODevice::Text);
-    Q_ASSERT(b);
-    f.write(QByteArrayLiteral("<?xml version='1.0' encoding='UTF-8'?>\n"
-    "<!DOCTYPE X3D PUBLIC 'ISO//Web3D//DTD X3D 3.2//EN' 'http://www.web3d.org/specifications/x3d-3.2.dtd'>\n"
-    "<X3D profile='Interchange' version='3.2' xmlns:xsd='http://www.w3.org/2001/XMLSchema-instance' "
-        "xsd:noNamespaceSchemaLocation='http://www.web3d.org/specifications/x3d-3.2.xsd'>\n"
-        "<Scene>\n"));
-    for (int i = 0; i < m_model->rowCount(); ++i)
-    {
-        PlotItem *item = itemAt(i);
-
-        Surface *surf = dynamic_cast<Surface*>(item);
-        if (!surf || !surf->isVisible())
-            continue;
-
-        f.write(QByteArrayLiteral(
-            "<Shape>\n"
-                "<Appearance><Material diffuseColor='1 0 0' specularColor='0.8 0.7 0.5'/></Appearance>\n"
-                "<IndexedFaceSet solid='false' normalPerVertex='false' coordIndex='"));
-        f.write(fromNumbers(makeTriangles(surf->indexes())));
-        f.write(QByteArrayLiteral("'>\n"
-                    "<Coordinate point='"));
-        f.write(fromNumbers(surf->vertices()));
-        f.write(QByteArrayLiteral("'/>\n"
-        "<Normal vector='"));
-        f.write(fromNumbers(surf->normals()));
-        f.write(QByteArrayLiteral(
-                "'/>\n"
-                "</IndexedFaceSet>\n"
-            "</Shape>\n"));
-    }
-    f.write(QByteArrayLiteral(
-        "</Scene>\n"
-    "</X3D>\n"));
+    if (path.endsWith(".stl", Qt::CaseInsensitive))
+        Export3D::exportSTL(path, m_model);
+    else
+        Export3D::exportX3D(path, m_model);
 }
 
 void Plotter3D::updatePlots(const QModelIndex & parent, int s, int e)
